@@ -1,7 +1,13 @@
-/* radare2 - LGPL - Copyright 2009-2024 - pancake */
+/* radare2 - LGPL - Copyright 2009-2025 - pancake */
 
 #if R_INCLUDE_BEGIN
 
+static RCoreHelpMessage help_msg_ex = {
+	"Usage ex", "[it|port]", " posix shell aliases only",
+	"exit", " [v]", "alias for 'q'",
+	"export", "[k=[v]]", "alias for '%'",
+	NULL
+};
 static RCoreHelpMessage help_msg_ecH = {
 	"Usage ecH[iw-?]", "", "",
 	"ecHi", "[color]", "highlight current instruction with 'color' background",
@@ -26,14 +32,15 @@ static RCoreHelpMessage help_msg_e = {
 #endif
 	"e.", "a=b", "same as 'e a=b' but without using a space",
 	"e,", "[table-query]", "show the output in table format",
-	"e/", "asm", "filter configuration variables by name",
 	"e:", "k=v:k=v:k=v", "comma or colon separated k[=v]",
 	"e-", "", "reset config vars",
 	"e*", "", "dump config vars in r commands",
 	"e!", "a", "invert the boolean value of 'a' var",
 	"ec", "[?] [k] [color]", "set color for given key (prompt, offset, ...)",
 	"ee", " [var]", "open cfg.editor to change the value of var",
-	"ed", "", "open editor to change the ~/.radare2rc",
+	"ed", " ([file])", "open editor to change the ~/.radare2rc or [file]",
+	"ed*", "", "show contents of your ~/.radare2c",
+	"ed+", "", "add or set an eval config line into your ~/.radare2c",
 	"ed-", "[!]", "delete ~/.radare2c (Use ed-! to delete without prompting)",
 	"ej", "", "list config vars in JSON",
 	"eJ", "", "list config vars in verbose JSON",
@@ -42,6 +49,7 @@ static RCoreHelpMessage help_msg_e = {
 	"er", " [key]", "set config key as readonly. no way back",
 	"es", " [space]", "list all eval spaces [or keys]",
 	"et", " [key]", "show type of given config variable",
+	"ex", "[?]", "prefix for posix shells like exit and export",
 	"ev", " [key]", "list config vars in verbose format",
 	"evj", " [key]", "list config vars in verbose format in JSON",
 	NULL
@@ -86,7 +94,7 @@ static RCoreHelpMessage help_msg_eco = {
 static void cmd_eval_table(RCore *core, const char *input) {
 	const char fmt = *input;
 	const char *q = input;
-	RTable *t = r_core_table (core, "eval");
+	RTable *t = r_core_table_new (core, "eval");
 	RTableColumnType *typeString = r_table_type ("string");
 	RTableColumnType *typeBoolean = r_table_type ("bool");
 	r_table_add_column (t, typeBoolean, "ro", 0);
@@ -106,7 +114,7 @@ static void cmd_eval_table(RCore *core, const char *input) {
 		char *s = (fmt == 'j')
 			? r_table_tojson (t)
 			: r_table_tostring (t);
-		r_cons_printf ("%s\n", s);
+		r_cons_printf (core->cons, "%s\n", s);
 		free (s);
 	}
 	r_table_free (t);
@@ -122,7 +130,7 @@ static bool nextpal_item(RCore *core, PJ *pj, int mode, const char *file) {
 		pj_s (pj, fn);
 		break;
 	case 'l': // list
-		r_cons_println (fn);
+		r_cons_println (core->cons, fn);
 		break;
 	case 'p': // previous
 		// TODO: move logic here
@@ -161,8 +169,9 @@ static char *get_theme_path(RCore *core, const char *theme_name) {
 	}
 	free (theme_path);
 	// check system directory
-	const char *r2pfx = r_sys_prefix (NULL);
+	char *r2pfx = r_sys_prefix (NULL);
 	theme_path = r_file_new (r2pfx, R2_THEMES, theme_name, NULL);
+	free (r2pfx);
 	if (r_file_exists (theme_path)) {
 		return theme_path;
 	}
@@ -199,15 +208,20 @@ static bool cmd_load_theme(RCore *core, const char *_arg) {
 			free (core->theme);
 			core->theme = strdup (_arg);
 		}
-		r_cons_pal_init (core->cons->context);
+		r_cons_pal_init (core->cons);
 		return true;
 	}
 	bool ret = false;
 	char *theme_script = get_theme_script (core, _arg);
 	if (R_STR_ISNOTEMPTY (theme_script)) {
 		core->cmdfilter = "ec ";
+		// Enable batch mode to skip individual pal_reload calls
+		core->cons->context->pal_batch = true;
 		r_core_cmd_lines (core, theme_script);
-		r_cons_pal_update_event ();
+		core->cons->context->pal_batch = false;
+		// Single reload after all color changes
+		r_cons_pal_reload (core->cons);
+		core->cons->context->pal_dirty = false;
 		core->cmdfilter = NULL;
 		ret = true; // maybe the script fails?
 	} else {
@@ -255,7 +269,7 @@ R_API RList *r_core_list_themes(RCore *core) {
 }
 
 static void nextpal(RCore *core, int mode) {
-// TODO: use r_core_list_themes() here instead of rewalking all the time
+	// TODO: use r_core_list_themes () here instead of rewalking all the time
 	RList *files = NULL;
 	RListIter *iter;
 	const char *fn;
@@ -347,7 +361,7 @@ done:
 		return;
 	}
 	if (mode == 'l' && !core->theme && !r_list_empty (files)) {
-		//nextpal (core, mode);
+		// nextpal (core, mode);
 	} else if (mode == 'n' || mode == 'p') {
 		if (R_STR_ISNOTEMPTY (core->theme)) {
 			r_core_cmd_callf (core, "eco %s", core->theme);
@@ -357,7 +371,7 @@ done:
 	files = NULL;
 	if (mode == 'j') {
 		pj_end (pj);
-		r_cons_println (pj_string (pj));
+		r_cons_println (core->cons, pj_string (pj));
 		pj_free (pj);
 	}
 }
@@ -365,16 +379,16 @@ done:
 R_API void r_core_echo(RCore *core, const char *input) {
 	if (r_str_startswith (input, "64 ")) {
 		char *buf = strdup (input);
-		r_base64_decode ((ut8*)buf, input + 3, -1);
+		r_base64_decode ((ut8 *)buf, input + 3, -1);
 		if (*buf) {
-			r_cons_echo (buf);
+			r_cons_echo (core->cons, buf);
 		}
 		free (buf);
 	} else {
 		char *p = strchr (input, ' ');
 		if (p) {
-			r_cons_print (p + 1);
-			r_cons_newline ();
+			r_cons_print (core->cons, p + 1);
+			r_cons_newline (core->cons);
 		}
 	}
 }
@@ -394,7 +408,7 @@ static bool is_static_theme(const char *th) {
 static bool cmd_ec(RCore *core, const char *input) {
 	switch (input[1]) {
 	case 'd': // "ecd"
-		r_cons_pal_init (core->cons->context);
+		r_cons_pal_init (core->cons);
 		break;
 	case '?':
 		r_core_cmd_help (core, help_msg_ec);
@@ -409,100 +423,100 @@ static bool cmd_ec(RCore *core, const char *input) {
 			}
 			break;
 		case '*': // "eco*"
-			{
-				const char *theme_name = core->theme;
-				if (input[3]) {
-					theme_name = r_str_trim_head_ro (input + 3);
-				}
-				char *theme_script = get_theme_script (core, theme_name);
-				if (R_STR_ISNOTEMPTY (theme_script)) {
-					r_cons_printf ("%s\n", theme_script);
-				} else {
-					R_LOG_ERROR ("Cannot find theme '%s'", theme_name);
-				}
-				free (theme_script);
+		{
+			const char *theme_name = core->theme;
+			if (input[3]) {
+				theme_name = r_str_trim_head_ro (input + 3);
 			}
-			break;
+			char *theme_script = get_theme_script (core, theme_name);
+			if (R_STR_ISNOTEMPTY (theme_script)) {
+				r_cons_printf (core->cons, "%s\n", theme_script);
+			} else {
+				R_LOG_ERROR ("Cannot find theme '%s'", theme_name);
+			}
+			free (theme_script);
+		}
+		break;
 		case '!':
 			free (r_core_editor (core, core->themepath, NULL));
 			cmd_load_theme (core, core->theme); // reload
 			break;
 		case ' ':
-			cmd_load_theme (core, input + 3);
+			cmd_load_theme (core, r_str_trim_head_ro (input + 3));
 			break;
 		case 'o':
 			cmd_load_theme (core, core->theme);
 			break;
 		case 'c':
 		case '.':
-			r_cons_printf ("%s\n", core->theme);
+			r_cons_printf (core->cons, "%s\n", core->theme);
 			break;
 		case '?':
 			r_core_cmd_help (core, help_msg_eco);
 			break;
 		default:
 			{
-			RList *themes_list = r_core_list_themes (core);
-			RListIter *th_iter;
-			const char *th;
-			const RConsTheme *themes = r_cons_themes ();
-			const RConsTheme *theme = themes;
+				RList *themes_list = r_core_list_themes (core);
+				RListIter *th_iter;
+				const char *th;
+				const RConsTheme *themes = r_cons_themes ();
+				const RConsTheme *theme = themes;
 			while (theme && theme->name) {
-				const char *th = theme->name;
-				if (input[2] == 'q') {
-					r_cons_printf ("%s\n", th);
+					const char *th = theme->name;
+					if (input[2] == 'q') {
+						r_cons_printf (core->cons, "%s\n", th);
 				} else if (core->theme && !strcmp (core->theme, th)) {
-					r_cons_printf ("- %s\n", th);
-				} else {
-					r_cons_printf ("  %s\n", th);
+						r_cons_printf (core->cons, "- %s\n", th);
+					} else {
+						r_cons_printf (core->cons, "  %s\n", th);
+					}
+					theme++;
 				}
-				theme++;
-			}
-			r_list_foreach (themes_list, th_iter, th) {
-				if (is_static_theme (th)) {
-					continue;
-				}
-				if (input[2] == 'q') {
-					r_cons_printf ("%s\n", th);
+				r_list_foreach (themes_list, th_iter, th) {
+					if (is_static_theme (th)) {
+						continue;
+					}
+					if (input[2] == 'q') {
+						r_cons_printf (core->cons, "%s\n", th);
 				} else if (core->theme && !strcmp (core->theme, th)) {
-					r_cons_printf ("- %s\n", th);
-				} else {
-					r_cons_printf ("  %s\n", th);
+						r_cons_printf (core->cons, "- %s\n", th);
+					} else {
+						r_cons_printf (core->cons, "  %s\n", th);
+					}
 				}
-			}
-			r_list_free (themes_list);
+				r_list_free (themes_list);
 			}
 			break;
 		}
 		break;
 	case 's': // "ecs"
-		r_cons_pal_show ();
+		r_cons_pal_show (core->cons);
 		break;
 	case '*': // "ec*"
-		r_cons_pal_list (1, NULL);
+		r_cons_pal_list (core->cons, 1, NULL);
 		break;
 	case 'h': // echo
 		if (input[2] == 'o') {
 			r_core_echo (core, input + 3);
 		} else {
-			r_cons_pal_list ('h', NULL);
+			r_cons_pal_list (core->cons, 'h', NULL);
 		}
 		break;
 	case 'j': // "ecj"
-		r_cons_pal_list ('j', NULL);
+		r_cons_pal_list (core->cons, 'j', NULL);
 		break;
 	case 'c': // "ecc"
 		if (input[2]) {
-			r_cons_pal_list ('c', input + 2);
+			r_cons_pal_list (core->cons, 'c', input + 2);
 		} else {
-			r_cons_pal_list ('c', r_config_get (core->config, "scr.css.prefix"));
+			r_cons_pal_list (core->cons, 'c', r_config_get (core->config, "scr.css.prefix"));
 		}
 		break;
 	case '\0': // "ec"
-		r_cons_pal_list (0, NULL);
+		r_cons_pal_list (core->cons, 0, NULL);
 		break;
 	case 'r': // "ecr"
-		r_cons_pal_random ();
+		r_cons_pal_random (core->cons);
 		break;
 	case 'n': // "ecn"
 		nextpal (core, 'n');
@@ -510,115 +524,121 @@ static bool cmd_ec(RCore *core, const char *input) {
 	case 'p': // "ecp"
 		nextpal (core, 'p');
 		break;
-	case 'H': { // "ecH"
-			  char *color_code = NULL;
-			  char *word = NULL;
-			  int argc = 0;
-			  int delta = (input[2])? 3: 2;
-			  char** argv = r_str_argv (r_str_trim_head_ro (input + delta), &argc);
-			  switch (input[2]) {
-			  case '?':
-				  r_core_cmd_help (core, help_msg_ecH);
-				  r_str_argv_free (argv);
-				  return false;
-			  case '-': // ecH-
-				  if (input[3] == '*') {
-					  r_meta_del (core->anal, R_META_TYPE_HIGHLIGHT, 0, UT64_MAX);
-				  } else {
-					  r_meta_del (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, 1);
-					  // r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, "");
-				  }
-				  r_str_argv_free (argv);
-				  return false;
-			  case '.':
-				  r_meta_print_list_in_function (core->anal, R_META_TYPE_HIGHLIGHT, 0, core->offset, NULL);
-				  r_str_argv_free (argv);
-				  return false;
-			  case '\0':
-				  r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 0, NULL);
-				  r_str_argv_free (argv);
-				  return false;
-			  case 'j':
-				  r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 'j', NULL);
-				  r_str_argv_free (argv);
-				  return false;
-			  case '*':
-				  r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, '*', NULL);
-				  r_str_argv_free (argv);
-				  return false;
-			  case ' ':
-			  case 'i': // "ecHi"
-				  if (argc) {
-					  char *dup = r_str_newf ("bgonly %s", argv[0]);
-					  color_code = r_cons_pal_parse (dup, NULL);
-					  R_FREE (dup);
-					  if (!color_code) {
-						  R_LOG_ERROR ("Unknown color %s", argv[0]);
-						  r_str_argv_free (argv);
-						  return true;
-					  }
-				  }
-				  break;
-			  case 'w': // "ecHw"
-				  if (!argc) {
-					  r_core_cmd_help_match (core, help_msg_ecH, "ecHw");
-					  r_str_argv_free (argv);
-					  return true;
-				  }
-				  word = strdup (argv[0]);
-				  if (argc > 1) {
-					  char *dup = r_str_newf ("bgonly %s", argv[1]);
-					  color_code = r_cons_pal_parse (dup, NULL);
-					  R_FREE (dup);
-					  if (!color_code) {
-						  R_LOG_ERROR ("Unknown color %s", argv[1]);
-						  r_str_argv_free (argv);
-						  free (word);
-						  return true;
-					  }
-				  }
-				  break;
-			  default:
-				  R_LOG_INFO ("See ecH?");
-				  r_str_argv_free (argv);
-				  return true;
-			  }
-			  r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, "");
-			  const char *str = r_meta_get_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset);
-			  char *dup = r_str_newf ("%s \"%s%s\"", r_str_get (str), r_str_get (word),
-					  color_code ? color_code : r_cons_singleton ()->context->pal.wordhl);
-			  r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, dup);
-			  r_str_argv_free (argv);
-			  free (color_code);
-			  R_FREE (word);
-			  R_FREE (dup);
-		  }
-		  break;
+	case 'H': // "ecH"
+	{
+		char *color_code = NULL;
+		char *word = NULL;
+		int argc = 0;
+		int delta = (input[2])? 3: 2;
+		char **argv = r_str_argv (r_str_trim_head_ro (input + delta), &argc);
+		switch (input[2]) {
+		case '?':
+			r_core_cmd_help (core, help_msg_ecH);
+			r_str_argv_free (argv);
+			return false;
+		case '-': // ecH-
+			if (input[3] == '*') {
+				r_meta_del (core->anal, R_META_TYPE_HIGHLIGHT, 0, UT64_MAX);
+			} else {
+				r_meta_del (core->anal, R_META_TYPE_HIGHLIGHT, core->addr, 1);
+				// r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->addr, "");
+			}
+			r_str_argv_free (argv);
+			return false;
+		case '.':
+			r_meta_print_list_in_function (core->anal, R_META_TYPE_HIGHLIGHT, 0, core->addr, NULL, NULL);
+			r_str_argv_free (argv);
+			return false;
+		case '\0':
+			r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 0, NULL, NULL);
+			r_str_argv_free (argv);
+			return false;
+		case 'j':
+			r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 'j', NULL, NULL);
+			r_str_argv_free (argv);
+			return false;
+		case '*':
+			r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, '*', NULL, NULL);
+			r_str_argv_free (argv);
+			return false;
+		case ' ':
+		case 'i': // "ecHi"
+			if (argc) {
+				char *dup = r_str_newf ("bgonly %s", argv[0]);
+				color_code = r_cons_pal_parse (core->cons, dup, NULL);
+				free (dup);
+				if (!color_code) {
+					R_LOG_ERROR ("Unknown color %s", argv[0]);
+					r_str_argv_free (argv);
+					return true;
+				}
+			}
+			break;
+		case 'w': // "ecHw"
+			if (!argc) {
+				r_core_cmd_help_match (core, help_msg_ecH, "ecHw");
+				r_str_argv_free (argv);
+				return true;
+			}
+			word = strdup (argv[0]);
+			if (argc > 1) {
+				char *dup = r_str_newf ("bgonly %s", argv[1]);
+				color_code = r_cons_pal_parse (core->cons, dup, NULL);
+				R_FREE (dup);
+				if (!color_code) {
+					R_LOG_ERROR ("Unknown color %s", argv[1]);
+					r_str_argv_free (argv);
+					free (word);
+					return true;
+				}
+			}
+			break;
+		default:
+			R_LOG_INFO ("See ecH?");
+			r_str_argv_free (argv);
+			return true;
+		}
+		r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->addr, "");
+		const char *str = r_meta_get_string (core->anal, R_META_TYPE_HIGHLIGHT, core->addr);
+		char *dup = r_str_newf ("%s \"%s%s\"", r_str_get (str), r_str_get (word),
+			color_code? color_code: core->cons->context->pal.wordhl);
+		r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->addr, dup);
+		r_str_argv_free (argv);
+		free (color_code);
+		R_FREE (word);
+		R_FREE (dup);
+	}
+	break;
 	case ' ':
-		  {
-			 char *p = strdup (input + 2);
-			 char *q = strchr (p, '=');
-			 if (!q) {
-				 q = strchr (p, ' ');
-			 }
-			 if (q) {
-				 // Set color
-				 *q++ = 0;
-				 if (r_cons_pal_set (p, q)) {
-					 r_cons_pal_update_event ();
-				 }
-			 } else {
-				 char color[32] = {0};
-				 RColor rcolor = r_cons_pal_get (p);
-				 r_cons_rgb_str (color, sizeof (color), &rcolor);
-				 if (*color) {
-					 eprintf ("(%s)(%sCOLOR"Color_RESET")\n", p, color);
-				 } else {
-					 R_LOG_ERROR ("Invalid palette color '%s'", p);
-				 }
-			 }
-			 free (p);
-		 }
+		{
+			char *p = strdup (input + 2);
+			char *q = strchr (p, '=');
+			if (!q) {
+				q = strchr (p, ' ');
+			}
+			if (q) {
+				// Set color
+				*q++ = 0;
+				if (r_cons_pal_set (core->cons, p, q)) {
+					core->cons->context->pal_dirty = true;
+					if (!core->cons->context->pal_batch) {
+						r_cons_pal_reload (core->cons);
+						// if we uncomment this changing color theme doesnt work from rcore_cmd_str
+						// core->cons->context->pal_dirty = false;
+					}
+				}
+			} else {
+				char color[32] = { 0 };
+				RColor rcolor = r_cons_pal_get (core->cons, p);
+				r_cons_rgb_str (core->cons, color, sizeof (color), &rcolor);
+				if (*color) {
+					eprintf ("(%s)(%sCOLOR" Color_RESET ")\n", p, color);
+				} else {
+					R_LOG_ERROR ("Invalid palette color '%s'", p);
+				}
+			}
+			free (p);
+		}
 		break;
 	default:
 		r_core_return_invalid_command (core, "ec", input[1]);
@@ -627,31 +647,109 @@ static bool cmd_ec(RCore *core, const char *input) {
 	return true;
 }
 
+static void r2rc_set(RCore *core, const char *R_NULLABLE k, const char *R_NULLABLE v) {
+	char *rcfile = r_file_home (".radare2rc");
+	char *rcdata = r_file_slurp (rcfile, NULL);
+	if (k) {
+		char *line;
+		RListIter *iter;
+		RList *lines = r_str_split_list (rcdata, "\n", 0);
+		RStrBuf *sb = r_strbuf_new ("");
+		bool found = false;
+		char *kk = r_str_newf ("e %s", k);
+		r_list_foreach (lines, iter, line) {
+			const char *oline = line;
+			if (*oline == '\'') {
+				oline++;
+			}
+			if (r_str_startswith (oline, kk)) {
+				if (v) {
+					if (!found && *v) {
+						r_strbuf_appendf (sb, "'e %s=%s\n", k, v);
+					}
+				} else {
+					r_cons_println (core->cons, line);
+				}
+				found = true;
+			} else {
+				r_strbuf_appendf (sb, "%s\n", line);
+			}
+		}
+		free (kk);
+		if (v) {
+			if (!found && *v) {
+				r_strbuf_appendf (sb, "'e %s=%s\n", k, v);
+			}
+			char *out = r_strbuf_drain (sb);
+			r_list_free (lines);
+			r_str_trim (out);
+			r_file_dump (rcfile, (const ut8 *)out, -1, false);
+			free (out);
+		} else {
+			r_strbuf_free (sb);
+		}
+	} else {
+		r_cons_println (core->cons, rcdata);
+	}
+	free (rcdata);
+	free (rcfile);
+}
+
+static void cmd_eplus(RCore *core, const char *input) {
+	char *s = r_str_trim_dup (input);
+	char *eq = strchr (s, '=');
+	if (*s) {
+		if (eq) {
+			r_str_trim (s);
+			*eq++ = 0;
+			r_str_trim (eq);
+			const char *k = s;
+			const char *v = eq;
+			r2rc_set (core, k, v);
+		} else {
+			r2rc_set (core, s, NULL);
+		}
+	} else {
+		r2rc_set (core, NULL, NULL);
+	}
+	free (s);
+}
+
+static void core_config_list(RCore *core, const char *str, int rad) {
+	char *res = r_config_list (core->config, str, rad);
+	r_cons_print (core->cons, res);
+	free (res);
+}
+
+static void core_config_eval(RCore *core, const char *input, bool uhm) {
+	char *res = r_config_eval (core->config, input, uhm, NULL);
+	r_cons_print (core->cons, res);
+	free (res);
+}
+
 static int cmd_eval(void *data, const char *input) {
 	RCore *core = (RCore *)data;
+	RCons *cons = core->cons;
 	switch (input[0]) {
 	case '\0': // "e"
-		r_config_list (core->config, NULL, 0);
+		core_config_list (core, NULL, 0);
 		break;
 	case '?': // "e?"
 		switch (input[1]) {
 		case '\0': r_core_cmd_help (core, help_msg_e); break;
-		case '?': r_config_list (core->config, input + 2, 2); break;
-		default: r_config_list (core->config, input + 1, 3); break;
+		case '?': core_config_list (core, input + 2, 2); break;
+		default: core_config_list (core, input + 1, 3); break;
 		}
 		break;
-	default:
-		r_core_return_invalid_command (core, "e", *input);
-		break;
 	case 't': // "et"
 		if (input[1] == 'a') {
-			r_cons_printf ("%s\n", (r_num_rand (10) % 2)? "wen": "son");
+			r_cons_printf (cons, "%s\n", (r_num_rand (10) % 2)? "wen": "son");
 		} else if (input[1] == ' ' && input[2]) {
-			RConfigNode *node = r_config_node_get (core->config, input+2);
+			RConfigNode *node = r_config_node_get (core->config, input + 2);
 			if (node) {
 				const char *type = r_config_node_type (node);
 				if (type && *type) {
-					r_cons_println (type);
+					r_cons_println (cons, type);
 				}
 			}
 		} else {
@@ -662,7 +760,8 @@ static int cmd_eval(void *data, const char *input) {
 		if (strchr (input, '?')) {
 			r_core_cmd_help_contains (core, help_msg_e, "en");
 			break;
-		} else if (!strcmp (input + 1, "vj")) {
+		}
+		if (!strcmp (input + 1, "vj")) {
 			char **e = r_sys_get_environ ();
 			PJ *pj = r_core_pj_new (core);
 			pj_o (pj);
@@ -680,13 +779,13 @@ static int cmd_eval(void *data, const char *input) {
 			}
 			pj_end (pj);
 			char *s = pj_drain (pj);
-			r_cons_println (s);
+			r_cons_println (cons, s);
 			free (s);
 		} else if (!strcmp (input + 1, "v*")) {
 			char **e = r_sys_get_environ ();
 			if (e != NULL) {
 				while (*e) {
-					r_cons_printf ("%%%s\n", *e);
+					r_cons_printf (cons, "%%%s\n", *e);
 					e++;
 				}
 			}
@@ -697,13 +796,13 @@ static int cmd_eval(void *data, const char *input) {
 			}
 			char *p = r_sys_getenv (var);
 			if (p) {
-				r_cons_println (p);
+				r_cons_println (cons, p);
 				free (p);
-			} else {
+			} else if (!var || !*var) {
 				char **e = r_sys_get_environ ();
 				if (e != NULL) {
 					while (*e) {
-						r_cons_println (*e);
+						r_cons_println (cons, *e);
 						e++;
 					}
 				}
@@ -729,56 +828,100 @@ static int cmd_eval(void *data, const char *input) {
 		}
 		return true;
 	case 'x': // "ex"
-		// XXX we need headers for the cmd_xxx files.
-		return cmd_quit (data, "");
+		if (input[1] == '?') {
+			r_core_cmd_help (core, help_msg_ex);
+			break;
+		}
+		if (r_str_startswith (input, "xit")) { // "exit"
+			return cmd_quit (data, r_str_trim_head_ro (input + strlen ("xit")));
+		}
+		if (r_str_startswith (input, "xport")) { // "export"
+			const char *arg = r_str_trim_head_ro (input + strlen ("xport"));
+			return r_core_cmdf (core, "%%%s", arg);
+		}
+		r_core_return_invalid_command (core, "ex", input[1]);
+		break;
 	case 'J': // "eJ"
-		r_config_list (core->config, NULL, 'J');
+		core_config_list (core, NULL, 'J');
 		break;
 	case 'j': // "ej"
-		r_config_list (core->config, NULL, 'j');
+		core_config_list (core, NULL, 'j');
 		break;
 	case 'v': // verbose
-		r_config_list (core->config, r_str_trim_head_ro (input + 1), 'v');
+		core_config_list (core, r_str_trim_head_ro (input + 1), 'v');
 		break;
 	case 'q': // quiet list of eval keys
-		r_config_list (core->config, NULL, 'q');
+		core_config_list (core, NULL, 'q');
 		break;
 	case 'c': // "ec"
 		return cmd_ec (core, input);
 	case 'd': // "ed"
 		if (input[1] == '?') {
 			r_core_cmd_help_contains (core, help_msg_e, "ed");
+		} else if (input[1] == '!') {
+			char *file = r_file_home (".radare2rc");
+			free (r_cons_editor (cons, file, NULL));
+			free (file);
+		} else if (input[1] == '*') {
+			char *file = r_file_home (".radare2rc");
+			char *data = r_file_slurp (file, NULL);
+			r_cons_println (cons, data);
+			free (data);
+			free (file);
+		} else if (input[1] == '+') {
+			cmd_eplus (core, input + 2);
 		} else if (input[1] == '-') { // "ed-"
 			const bool prompt = (input[2] != '!');
 			char *file = r_file_home (".radare2rc");
 			if (file) {
-				const bool rmfile = !prompt || r_cons_yesno ('n', "Do you want to delete ~/.radare2? (Y/n)");
+				const bool rmfile = !prompt || r_cons_yesno (cons, 'n', "Do you want to delete ~/.radare2? (Y/n)");
 				if (rmfile) {
 					r_file_rm (file);
 				}
 				free (file);
 			}
 		} else {
-			char *file = r_file_home (".radare2rc");
-			if (r_cons_is_interactive ()) {
-				r_file_touch (file);
-				char *res = r_cons_editor (file, NULL);
-				if (res) {
-					if (r_cons_yesno ('y', "Reload? (Y/n)")) {
-						r_core_run_script (core, file);
-					}
+			const char *arg = strchr (input, ' ');
+			char *file = NULL;
+			bool is_config = true;
+			if (arg) {
+				arg = r_str_trim_head_ro (arg + 1);
+				if (arg[0] == '~') {
+					file = r_file_home (arg + 1);
+				} else {
+					file = strdup (arg);
 				}
+				is_config = false;
 			} else {
-				r_core_run_script (core, file);
+				file = r_file_home (".radare2rc");
+			}
+			if (file) {
+				if (r_cons_is_interactive (cons)) {
+					if (is_config) {
+						r_file_touch (file);
+					}
+					char *res = r_cons_editor (cons, file, NULL);
+					if (res && is_config) {
+						if (r_cons_yesno (cons, 'y', "Reload? (Y/n)")) {
+							r_core_run_script (core, file);
+						}
+					}
+					free (res);
+				} else if (is_config) {
+					r_core_run_script (core, file);
+				}
 			}
 			free (file);
 		}
 		break;
+	case '+': // "e+"
+		cmd_eplus (core, input + 1);
+		break;
 	case 'e': // "ee"
 		if (input[1] == ' ') {
 			char *p;
 			const char *input2 = strchr (input + 2, ' ');
-			input2 = (input2) ? input2 + 1 : input + 2;
+			input2 = (input2)? input2 + 1: input + 2;
 			const char *val = r_config_get (core->config, input2);
 			p = r_core_editor (core, NULL, val);
 			if (p) {
@@ -800,14 +943,14 @@ static int cmd_eval(void *data, const char *input) {
 		}
 		break;
 	case 's': // "es"
-		r_config_list (core->config, (input[1])? input + 1: NULL, 's');
+		core_config_list (core, (input[1])? input + 1: NULL, 's');
 		break;
 	case '-': // "e-"
 		r_core_config_init (core);
-		//eprintf ("BUG: 'e-' command locks the eval hashtable. patches are welcome :)\n");
+		// eprintf ("BUG: 'e-' command locks the eval hashtable. patches are welcome :)\n");
 		break;
 	case '*': // "e*"
-		r_config_list (core->config, NULL, 1);
+		core_config_list (core, NULL, 1);
 		break;
 	case 'r': // "er"
 		if (input[1]) {
@@ -820,7 +963,7 @@ static int cmd_eval(void *data, const char *input) {
 		}
 		break;
 	case ':': // "e:"
-		r_config_eval (core->config, input + 1, true);
+		core_config_eval (core, input + 1, true);
 		break;
 	case ',': // "e,"
 		cmd_eval_table (core, input + 1);
@@ -828,20 +971,23 @@ static int cmd_eval(void *data, const char *input) {
 	case '.': // "e "
 	case ' ': // "e "
 		if (strchr (input, '=')) {
-			r_config_eval (core->config, r_str_trim_head_ro (input + 1), false);
+			core_config_eval (core, r_str_trim_head_ro (input + 1), false);
 		} else {
 			if (r_str_endswith (input, ".") && !r_str_endswith (input, "..")) {
-				r_config_list (core->config, input + 1, 0);
+				core_config_list (core, input + 1, 0);
 			} else if (r_str_endswith (input, ".?")) {
 				char *w = r_str_ndup (input, strlen (input) - 1);
-				r_config_list (core->config, w, 2);
+				core_config_list (core, w, 2);
 				free (w);
 			} else {
 				// XXX we cant do "e cmd.gprompt=dr=", because the '=' is a token, and quotes dont affect him
-				r_config_eval (core->config, r_str_trim_head_ro (input + 1), false);
+				core_config_eval (core, r_str_trim_head_ro (input + 1), false);
 			}
 		}
 		break;
+	default:
+		r_core_return_invalid_command (core, "e", *input);
+		break;
 	}
 	return 0;
 }
